<?php
/***************************************************************
*  Copyright notice
* 
*  (c) 1999-2003 Kasper Skrhj (kasper@typo3.com)
*  All rights reserved
* 
*  This script is part of the Typo3 project. Typo3 is free software;
*  You can redistribute it and/or modify it under the terms of the 
*  Typo3 License as published from the www.typo3.com website.
* 
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
* 
*  This copyright notice MUST APPEAR in all copies of this script
***************************************************************/
/** 
 * Typo3 Backend Language class
 *
 * The lang-class is used to define the labels in the various languages. 
 * This is loaded when Typo3's interface is started and these messages are included once and copied into the JavaScript
 * If you translate Typo3 into another language you MUST be shure to include all settings in this document.
 * 
 * Translation is done by creating a new folder is this folder. Put a "conf.php" in this folder an let it contain a $mainLang-array 
 * corresponding to the one in this class. You should definitely look at the current example of the danish translation...
 * 
 * @author	Kasper Skrhj <kasper@typo3.com>
 */

 

class language {
	var $lang="default";			// This is set to the language that is currently running for the user
	var $langSplit="default";	// Values like the labels in the tables.php-document are split by "|". This values defines which language is represented by which position in the resulting array after splitting a value.

	var $typo3_help_url= "";			// This is the url to the Typo3 manual
	var $charSet = "";						// String. Charset other than 'iso-8859-1' to use in the backend, eg. 'windows-1250'
	var $mainLang = Array();
	var $php3Lang = Array();

		// Internal
	var $langSplitIndex=0;
	var $LL_files_cache=array();
	var $LL_labels_cache=array();

	function init($lang,$altPath="")	{	
#debug($lang);
		$this->loadDefault($altPath?$altPath:PATH_typo3);

		// initializes the language $lang. 
		// The folder is found and the conf.php file is included. The language-overriding is done.
		$this->langSplit=TYPO3_languages;
		$path=($altPath?$altPath:PATH_typo3)."sysext/lang/".$lang;
		if (@is_dir($path) && @file_exists($path."/conf.php"))	{
			$this->lang = $lang;
			include($path."/conf.php");		// The conf-file is included. This must be valid PHP.
			$this->addLanguage($mainLang,"");	// $mainLang-array from the included file is loaded
			$this->addLanguage_PHP($php3Lang,"");	// $mainLang-array from the included file is loaded
			if ($typo3_help_url)	$this->typo3_help_url=$typo3_help_url;	// Help-url is set.
			if ($charSet)	$this->charSet=$charSet;	// Charset is set

			$ls = explode("|",$this->langSplit);
			while(list($i,$v)=each($ls))	{
				if ($v==$this->lang)	{$this->langSplitIndex=$i;}
			}
		}
//		$this->typo3_help_url = $GLOBALS["helpUrl"].$this->typo3_help_url;
	}
	function loadDefault($basepath)	{
		$path=$basepath."sysext/lang/default";
		include($path."/conf.php");		// The conf-file is included. This must be valid PHP.

		$this->mainLang = $mainLang;
		$this->mainLang["cm_images"] = Array (			// for each element in "cm" you can specify an icon, ?x12 pixels right-aligned
			"edit" => "gfx/edit2.gif",
			"view" => "gfx/zoom.gif",
			"info" => "gfx/zoom2.gif"
		);
		$this->mainLang["cm_preloadList"] = "view,edit,new,info,copy,cut,delete";
		$this->mainLang["tabs_images"] = Array (		// for each element in "tabs" you can specify an icon, ?x12 pixels left-aligned
			"normal" => ""
		);
		$this->mainLang["buttons_h"] = Array (		// for each element in "buttons": If a button is generated with a highligted version
			"clipboard" => "1",
			"logout" => "1",
			"clear" => "1"
		);
		$this->mainLang["buttons_images"] = Array (		// for each element in "buttons" you can specify an icon, ?x12 pixels left-aligned
			"clipboard" => "gfx/clipbrd.gif"
		);
		$this->mainLang["buttons_noJS"] = Array ();		// for each element in "buttons": True-values disables output to JS but just generates the button!!

#debug($this->mainLang);
		
		$this->php3Lang = $php3Lang;
		$this->typo3_help_url=$typo3_help_url;
		$this->charSet=$charSet;
	}
	function printClickmenuToJS()	{
		// This checks for and possibly generates the gif-files for the click-menues.
		// Afterwards the images are loaded by JavaScript statements
		if (is_array($this->mainLang[cm]))	{
			reset($this->mainLang[cm]);
			while(list($key,$val)=each($this->mainLang[cm])) {
				$name = $GLOBALS["TBE_iGraphics"]->makeClick($val,$this->mainLang[cm_images][$key]);
				$preLoad = t3lib_div::inList($this->mainLang[cm_preloadList],$key) ? 1 : 0;
				echo "
	glImage.add ('cm_".$key."', 104, 12, 'temp/".t3lib_div::slashJS($name)."','','','',".$preLoad.");";
			}
		}
		echo "
		";
	}
	function printTabsToJS()	{
		// This checks for and possibly generates the gif-files for the panes.
		// Afterwards the images are loaded by JavaScript statements
		if (is_array($this->mainLang[tabs]))	{
			reset($this->mainLang[tabs]);
			while(list($name,$label)=each($this->mainLang[tabs])) {
				if ($this->mainLang[tabs_images][$name] && @is_file($this->mainLang[tabs_images][$name]))	{
					$basefile = $this->mainLang[tabs_images][$name];
				} else {
					$basefile = "";
				}	
				if (!$basefile && !$label) {
					$label="...";
				}
				$modconf[icon_dims] = $GLOBALS["TBE_iGraphics"]->makePane($name,$label,$basefile);
				echo "
	glImage.add ('fane_".$name."', ".$modconf[icon_dims][0].", 12, 'temp/".$modconf[icon_dims][name]."');";
			}
		}
		echo "
		";
	}
	function printButtonsToJS($returnArray=0)	{
		// This checks for and possibly generates the gif-files for the buttons.
		// Afterwards the images are loaded by JavaScript statements
		$items=array();
		if (is_array($this->mainLang[buttons]))	{
			reset($this->mainLang[buttons]);
			while(list($name,$label)=each($this->mainLang[buttons])) {
				if ($this->mainLang[buttons_images][$name] && @is_file($this->mainLang[buttons_images][$name]))	{
					$basefile = $this->mainLang[buttons_images][$name];
				} else {
					$basefile = "";
				}	
				if (!$basefile && !$label) {
					$label="...";
				}
				$modconf[icon_dims] = $GLOBALS["TBE_iGraphics"]->makeButtons($name."_n",$label,$basefile,"");
				if ($this->mainLang[buttons_h][$name])	{	// Only if stated, we make a highlighted one
					$modconf2[icon_dims] = $GLOBALS["TBE_iGraphics"]->makeButtons($name."_h",$label,$basefile,"139,148,183");
				} else {
					$modconf2="";
				}
				if (!$this->mainLang["buttons_noJS"][$name])	{
					if (!$returnArray)	{
						echo "
	glImage.add ('button_".$name."', ".$modconf[icon_dims][0].", ".$modconf[icon_dims][1].", 'temp/".$modconf[icon_dims][name]."','','".($modconf2 ? "temp/".$modconf2[icon_dims][name] : "")."');";
					} else $items["button_".$name] = "temp/".$modconf[icon_dims][name];
				}
			}
		}
		if (!$returnArray)	{
			echo "
		";
		} else return $items;
	}
	function printJS()	{
		// This outputs the interface-labels as JavaScript code
		$this->printArrayToJS($this->mainLang["labels"],"l");
		$this->printArrayToJS($this->mainLang["mess"],"mess");
		$this->printArrayToJS($this->mainLang["rm"],"rm");
		$this->printArrayToJS($this->mainLang["err"],"err");
	}
	function printArrayToJS($arr,$pref)	{
		// internal...
		if (is_array($arr))	{
			reset($arr);
			while(list($key,$val)=each($arr)) {
				echo "
	langObj.add('".$pref."_".$key."','".t3lib_div::slashJS($val)."');";
			}
		}
		echo "
		";
	}
	function addLanguage($arr,$prefix)	{
		// This function takes an array corresponding to $this->mainLang.
		// The values in these arrays are transferred to $this->mainLang in this object and as such it provides a way to override the default language labels
		if (is_array($arr))	{
			reset($arr);
			while(list($k,$larr)=each($arr))	{
				if (is_array($larr))	{
					reset($larr);
					while(list($l,$v)=each($larr))	{
						if (isset($this->mainLang[$k]))	{
							$this->mainLang[$k][$prefix.$l]=$v;
						}
					}
				}
			}
		}
	}
	function addLanguage_PHP($arr,$prefix)	{
		// This function takes an array corresponding to $this->php3Lang.
		// The values in these arrays are transferred to $this->php3Lang in this object and as such it provides a way to override the default language labels
		if (is_array($arr))	{
			reset($arr);
			while(list($k,$larr)=each($arr))	{
				if (is_array($larr))	{
					reset($larr);
					while(list($l,$v)=each($larr))	{
						if (isset($this->php3Lang[$k]))	{
							$this->php3Lang[$k][$prefix.$l]=$v;
						}
					}
				}
			}
		}
	}
	function getLL($index)	{
		// Get Local Language
#		if (isset($GLOBALS["LOCAL_LANG"][$this->lang][$index]))	{
		if (strcmp($GLOBALS["LOCAL_LANG"][$this->lang][$index],""))	{
			return $GLOBALS["LOCAL_LANG"][$this->lang][$index];
		} else {
			return $GLOBALS["LOCAL_LANG"]["default"][$index];
		}
	}
	function getLLL($index,$LOCAL_LANG)	{
		// Get Local Language
#		if (isset($LOCAL_LANG[$this->lang][$index]))	{
		if (strcmp($LOCAL_LANG[$this->lang][$index],""))	{
			return $LOCAL_LANG[$this->lang][$index];
		} else {
			return $LOCAL_LANG["default"][$index];
		}
	}


	function sL($input)	{		// splitLabel
		if (strcmp(substr($input,0,4),"LLL:"))	{
	// Debug-output of places with language splitted stuff:
#if ($input && strstr($input,"|")) debug(array($input));

			$t = explode("|",$input);
			return $t[$this->langSplitIndex] ? $t[$this->langSplitIndex] : $t[0];
		} else {
			if (!isset($this->LL_labels_cache[$this->lang][$input])) {	// If cached label
				$restStr = trim(substr($input,4));
				$extPrfx="";
				if (!strcmp(substr($restStr,0,4),"EXT:"))	{
					$restStr = trim(substr($restStr,4));
					$extPrfx="EXT:";
				}
				$parts = explode(":",$restStr);
				$parts[0]=$extPrfx.$parts[0];
				if (!isset($this->LL_files_cache[$parts[0]]))	{	// Getting data if not cached
					$this->LL_files_cache[$parts[0]] = $this->readLLfile($parts[0]);
#debug(array("reading file" => $parts[0]));
				}
				$this->LL_labels_cache[$this->lang][$input] = $this->getLLL($parts[1],$this->LL_files_cache[$parts[0]]);
/*				if ($parts[2]=="ESQ")	{
					$this->LL_labels_cache[$this->lang][$input] = str_replace("'","\'",$this->LL_labels_cache[$this->lang][$input]);
				}
				*/
#debug(array("made:"=>$this->lang."/".$input));
			} else {
#debug(array("cached:"=>$this->lang."/".$input));
			}
			return $this->LL_labels_cache[$this->lang][$input];
		}
	}
	function readLLfile($fileRef)	{
		$file = t3lib_div::getFileAbsFileName($fileRef);
		if (@is_file($file))	{
			include($file);
		}
		return is_array($LOCAL_LANG)?$LOCAL_LANG:array();
	}
}

?>